; ===============================================================================================================================
; Projekt : kleines Testprogramm fr Backup und Restore mit 7-Zip :
; erstellt am 23.09.2017 von @Musashi
; ===============================================================================================================================
#Region ;**** Directives created by AutoIt3Wrapper_GUI ****
#AutoIt3Wrapper_icon=MyAutoIt3_Blue.ico
#AutoIt3Wrapper_UseX64 = N
#AutoIt3Wrapper_UseUpx = N ; interne UPX Komprimierung ausschalten
#AutoIt3Wrapper_Res_SaveSource = N
#AutoIt3Wrapper_Res_Description = Testprogramm fr Backup und Restore
#EndRegion ;**** Directives created by AutoIt3Wrapper_GUI ****

#include <GUIConstants.au3>
#include <Constants.au3>
#include <StringConstants.au3>
#include <Misc.au3>

Opt('MustDeclareVars', 1)  ; Variablen mssen vor Verwendung mit Global/Local deklariert werden

; Name der Backupdatei (voller Pfad) :
Global $sBackupFileName = @ScriptDir & "\backup.zip"

; Name der verwendeten ZIP-Software (voller Pfad) :
Global $sZipProgramm = @ScriptDir & "\7za.exe"  ; hier : 7za.exe im Skriptverzeichnis

; Hauptroutine aufrufen :
_GUI_Main()
Exit

; -------------------------------------------------------------------------------------------------------------------------------
; Funktion     : _GUI_Main
; -------------------------------------------------------------------------------------------------------------------------------
Func _GUI_Main()
	Local $iGuiMsg
	Local Const $iMinPosL = 30  ; Offset fr linke Startposition von GUI-Elementen

	; Controls GUI-Seite :
	Local $hGUI, $idLblInfo, $idProgressBar, $idBtnBackup, $idBtnRestore, $idBtnCancel

	$hGUI = GUICreate("Datensicherung : ", 500, 200, -1, -1)
	GUISetBkColor($COLOR_WHITE)
	GUISetFont(10, $FW_NORMAL, 1, "Arial")  ; Standardfont GUI-weit setzen

	; Infotext :
	$idLblInfo = GUICtrlCreateLabel("", $iMinPosL, 10, 400, 25)
	GUICtrlSetFont($idLblInfo, 12)
	GUICtrlSetState($idLblInfo, $GUI_HIDE)

	; ProgressBar initialisieren :
	Local $idProgressBar = GUICtrlCreateProgress($iMinPosL, 50, 440, 25, $PBS_MARQUEE)
	GUICtrlSetColor($idProgressBar , 0x426DA9)   ; Farbwert (bei Bedarf ndern)
	GUICtrlSetBkColor($idProgressBar, 0xF0F0F0)  ; Farbwert Hintergrund (bei Bedarf ndern)
	GUICtrlSetState($idProgressBar, $GUI_HIDE)

	$idBtnBackup  = GUICtrlCreateButton("&Backup", $iMinPosL, 150, 120, 25)
	$idBtnRestore = GUICtrlCreateButton("&Restore", $iMinPosL+140, 150, 120, 25)
	$idBtnCancel  = GUICtrlCreateButton("&Abbrechen", $iMinPosL+280, 150, 120, 25)

	GUISetState(@SW_SHOW, $hGUI)

	While 1
		$iGuiMsg = GUIGetMsg()

		Select
			Case $iGuiMsg = $GUI_EVENT_CLOSE
				ExitLoop

			Case $iGuiMsg = $idBtnBackup
				GUICtrlSetData($idLblInfo, "Datensicherung luft ... ")
				GUICtrlSetState($idLblInfo, $GUI_SHOW)
				GUICtrlSetState($idProgressBar, $GUI_SHOW)

				_ProgressMarquee_Start($idProgressBar)
				; Funktion fr Datensicherung aufrufen :
				_Backup()
				Sleep(5000)  ; zum Test : 5 Sek. Pause, um bei geringer Datenmenge die Progressbar zu sehen
				_ProgressMarquee_Stop($idProgressBar)
				ExitLoop

			Case $iGuiMsg = $idBtnRestore
				If Not FileExists($sBackupFileName) Then
					MsgBox(0, "Fehler :", $sBackupFileName & " nicht gefunden")
					ExitLoop
				EndIf

				GUICtrlSetData($idLblInfo, "Datenwiederherstellung luft ... ")
				GUICtrlSetState($idLblInfo, $GUI_SHOW)
				GUICtrlSetState($idProgressBar, $GUI_SHOW)

				_ProgressMarquee_Start($idProgressBar)
				; Funktion fr Datenwiederherstellung aufrufen :
				_Restore()
				Sleep(5000)  ; zum Test : 5 Sek. Pause, um bei geringer Datenmenge die Progressbar zu sehen
				_ProgressMarquee_Stop($idProgressBar)
				ExitLoop

			Case $iGuiMsg = $idBtnCancel
				ExitLoop

		EndSelect
	WEnd
    GUIDelete($hGUI)
EndFunc	;==>_GUI_Main

; -------------------------------------------------------------------------------------------------------------------------------
; Funktion     : _Backup
; Beschreibung : ausgewhlte Dateien mit 7za packen
; -------------------------------------------------------------------------------------------------------------------------------
Func _Backup()
	Local $sFileFolder  ; Datei oder Ordnername, der gepackt werden soll
	Local $sCommand     ; ZIP-Befehl (nur der bersichtlichkeit wegen)

	; eine Datei separat in das ZIP-Backup eintragen :
	$sFileFolder = @ScriptDir & "\SourceDir\File-SourceDir.txt"
	If FileExists($sFileFolder) Then
		$sCommand = $sZipProgramm & ' a -tzip "' & $sBackupFileName & '" "' & $sFileFolder & '"'
		RunWait ($sCommand, "", @SW_HIDE)
	Else
		MsgBox(0, "Fehler :", $sFileFolder & " nicht gefunden")
	EndIf

	; ein ausgewhltes Unterverzeichnis in das ZIP-Backup eintragen :
	$sFileFolder = @ScriptDir & "\SourceDir\SubDir1\"
	If FileExists($sFileFolder) Then
		$sCommand = $sZipProgramm & ' a -tzip "' & $sBackupFileName & '" "' & $sFileFolder & '"'
		RunWait ($sCommand, "", @SW_HIDE)
	Else
		MsgBox(0, "Fehler :", $sFileFolder & " nicht gefunden")
	EndIf

	; ein ausgewhltes Unterverzeichnis (mit Unterverzeichnissen) in das ZIP-Backup eintragen :
	$sFileFolder = @ScriptDir & "\SourceDir\SubDir2\"
	If FileExists($sFileFolder) Then
		$sCommand = $sZipProgramm & ' a -tzip "' & $sBackupFileName & '" "' & $sFileFolder & '"'
		RunWait ($sCommand, "", @SW_HIDE)
	Else
		MsgBox(0, "Fehler :", $sFileFolder & " nicht gefunden")
	EndIf
EndFunc   ;==>_Backup

; -------------------------------------------------------------------------------------------------------------------------------
; Funktion     : _Restore
; Beschreibung : Backupdatei mit 7za wieder entpacken (zurckspielen)
; -------------------------------------------------------------------------------------------------------------------------------
Func _Restore()
	Local $sSourceFolder  ; Zielordner, in den die ZIP-Struktur entpackt werden soll
	Local $sCommand       ; ZIP-Befehl (nur der bersichtlichkeit wegen)

	$sSourceFolder = @ScriptDir & "\SourceDir"

	If FileExists($sBackupFileName) And FileGetSize($sBackupFileName) > 0 Then
		; Info :
		;   -aoa => bestehende Dateien berschreiben
		;   -aos => bestehende Dateien berspringen
		$sCommand = $sZipProgramm & ' x "' & $sBackupFileName & '" -o"' & $sSourceFolder & '" -aoa -y'
		RunWait ($sCommand, "", @SW_HIDE)
	EndIf
EndFunc   ;==>_Restore

; -------------------------------------------------------------------------------------------------------------------------------
; Funktion     : _ProgressMarquee_Start
; Beschreibung : starten einer Progressanzeige im Marquee-Stil
; -------------------------------------------------------------------------------------------------------------------------------
Func _ProgressMarquee_Start($iControlID)
    Return GUICtrlSendMsg($iControlID, $PBM_SETMARQUEE, True, 30)  ; 30=Geschwindigkeit
EndFunc   ;==>_ProgressMarquee_Start

; -------------------------------------------------------------------------------------------------------------------------------
; Funktion     : _ProgressMarquee_Stop
; Beschreibung : stoppen der Progressanzeige
; -------------------------------------------------------------------------------------------------------------------------------
Func _ProgressMarquee_Stop($iControlID, $bReset = False)
    Local $bReturn = GUICtrlSendMsg($iControlID, $PBM_SETMARQUEE, False, 30)
    If $bReturn And $bReset Then
        GUICtrlSetStyle($iControlID, $PBS_MARQUEE)
    EndIf
    Return $bReturn
EndFunc   ;==>_ProgressMarquee_Stop